package it.html.oauth.oauthclient;

import android.app.ProgressDialog;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.abubusoft.kripton.android.Logger;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.io.IOException;
import java.util.Map;

import it.html.oauth.oauthclient.events.CompleteLoginEvent;
import it.html.oauth.oauthclient.model.User;

public class LoginActivity extends AppCompatActivity {

    private WebView mWebView;

    private EventBus bus = EventBus.getDefault();

    private ProgressDialog progressDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        bus = EventBus.getDefault();
        bus.register(this);

        progressDialog = new ProgressDialog(this);
        progressDialog.setIndeterminate(true);
        progressDialog.setMessage("Authenticating...");
        progressDialog.show();

        mWebView = (WebView) findViewById(R.id.webView);
        mWebView.clearCache(true);
        mWebView.getSettings().setJavaScriptEnabled(true);
        mWebView.getSettings().setBuiltInZoomControls(true);
        mWebView.getSettings().setDisplayZoomControls(false);
        mWebView.setWebViewClient(mWebViewClient);

        Logger.info("**** GOTO : %s", OAuthClientApplication.getUrlResolver().URL_LOGIN + OAuthClientApplication.getUrlResolver().ACTION_LOGIN);
        mWebView.loadUrl(OAuthClientApplication.getUrlResolver().URL_LOGIN);
    }

    @Override
    protected void onPause() {
        super.onPause();

        bus.unregister(this);
    }

    private WebViewClient mWebViewClient = new WebViewClient() {

        @Override
        public boolean shouldOverrideUrlLoading(WebView view, String url) {
            if (url.startsWith(OAuthClientApplication.getUrlResolver().URL_BASE) && url.contains(OAuthClientApplication.getUrlResolver().ACTION_CALLBACK)) {
                //ASSERT: siamo nell'ultimo pezzo
                mWebView.stopLoading();
                mWebView.setVisibility(View.INVISIBLE); // Hide webview if necessary

                url = url.replace(OAuthClientApplication.getUrlResolver().ACTION_CALLBACK, OAuthClientApplication.getUrlResolver().ACTION_USER);

                bus.post(new CompleteLoginEvent(url));
                return false;
            }
            return super.shouldOverrideUrlLoading(view, url);
        }

        @Override
        public void onPageFinished(WebView view, String url) {
            super.onPageFinished(view, url);

            if (progressDialog!=null) {
                progressDialog.dismiss();
                progressDialog=null;
            }
        }
    };

    @Subscribe(threadMode = ThreadMode.BACKGROUND)
    public void onEventComplete(CompleteLoginEvent event) throws IOException {
        Map<String, String> params = UrlHelper.extractQueryParams(event.getUrl());

        Logger.info("*** CODICE " + params.get("code"));

        User user = OAuthClientApplication.getLoginRestService().getUser(params.get("code")).execute().body();
        Logger.info("*** USER " + user.getEmail());

        BindOAuthClientSharedPreferences sp = BindOAuthClientSharedPreferences.instance();

        sp.refresh().edit().putUser(user).commit();

        Logger.info("*** LOGIN EFFETTUATO ");
        finish();
    }


}
